/* Minimum Cut Linear Arrangement */

/* Written by Qiong Cheng <qcheng1@student.gsu.edu> */

/* The Minimum Cut Linear Arrangement is stated as follows.
   Let a directed graph G = (V, E) be given, where V = {1, ..., n} is
   a set of nodes, E <= V x V is a set of arcs. Let also each arc
   e = (i,j) be assigned a number w[i,j], which is the weight of the
   arc e. The problem is to find a closed path of minimal weight cut which 
   meets maximum number of cut edges in any integer point. */

param n, integer, >= 3;
/* number of nodes */

param m, integer, >= 3;
/* number of edges */

set V := 0..n-1;
/* set of nodes */

set E, within V cross V;
/* set of arcs */

param W{(i,j) in E};
/* weight from node i to node j */

var M, integer, <= m;
/* maximum number of cut edges in any integer point (t) */

var F{i in 0..n-1}, integer, = ((i + 1) mod n) + 1;
/* define the transition function for every node */

var T{(i,j) in E, t in 1..n}, binary;#integer, >= 0, <= 1;
/* T[i,j, t] = 1 means that the edge from node i to node j is belonging to 
   maximum number of cut edges in any integer point(t) */

#minimize MinCut{t in 1..n}: sum{(i,j) in E} (W[i,j]*T[i,j,t]);
/* the objective is to make the weighted cut as small as possible */

#s.t. maxNumOfCutEdges{t in 1..n}: M[t] - (sum{(i,j) in E} T[i,j,t]) >= 0;
/* constraint: maximum number of cut edges in any integer point(t) */

maximize MinCut{t in 1..n}: sum{(i,j) in E} (W[i,j]*T[i,j,t]);
#s.t. cutEdge1{t in 1..n, (i,j) in E}: ((F[i]-t)-T[i,j,t]) >= 0 and ((t-F[j])-T[i,j,t]) >= 0;

#s.t. cutEdge2{t in 1..n, (i,j) in E}: ((t-F[j])-T[i,j,t]), >= 0;
/* constraint: For any edge(i,j), it meets F(i)<=t<=F(j) *(t-F[j])*/

#s.t. cutEdge2{t in 1..n, (i,j) in E}: if (F[j]-t> 0 and f[i]-t <= 0 and W[i, j] > 0) then T[i,j,t] = 1 else T[i,j,t] = 0;

s.t. cutEdge2{t in 1..n, (i,j) in E}: T[i,j,t], = (exists {(i,j) in E} t-F[j]< 0 and f[i]-t <= 0 and W[i, j] > 0);

solve;

printf{(i,j) in E} "      %3d       %3d   %8g\n",
   i, j, W[i,j];
   
printf{t in 1..n} "Maximum number of cut edges in any integer point(%d) : %d\n",
   t, sum{(i,j) in E} W[i,j] * T[i,j, t];
printf("Selected edge :\n");
printf{(i,j) in E, t in 1..n} if T[i,j, t] then "      %3d       %3d   %8g\n" else "",
   i, j, W[i,j];


